/* This file was generated by syncqt. */
#ifndef QT_QTSCXMLQML_VERSION_H
#define QT_QTSCXMLQML_VERSION_H

#define QTSCXMLQML_VERSION_STR "6.12.0"

#define QTSCXMLQML_VERSION 0x0601200

#endif // QT_QTSCXMLQML_VERSION_H
